/*this file modified by Tom Pratum, July 2007 - April 2009*/
/*
 This part of the code is based heavily on both the USBNotification and EZ-Loader examples that are part of Apple's IOKit*/

#include <unistd.h>

#include <CoreFoundation/CoreFoundation.h>

#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/usb/IOUSBLib.h>

#include <mach/mach.h>

#include "spinapi.h" /* this is where MAX_NUM_BOARDS is defined */

// globals
static IONotificationPortRef	gNotifyPort;
static io_iterator_t		gRawAddedIter;
static io_iterator_t		gRawRemovedIter;
static io_iterator_t		gNewDeviceAddedIter;
static io_iterator_t		gNewDeviceRemovedIter;

/*these device interfaces are populated when the devices are found initially*/
/*these are only used here and the global interface below is used in the spinapi functions*/
static IOUSBDeviceInterface **devg_h[MAX_NUM_BOARDS] = {NULL};

/*definition of usb interface array; note definition for 10.4 and above*/
static IOUSBInterfaceInterface300 	**intfg_h[MAX_NUM_BOARDS] = {NULL};

/*the device interfaces are set globally, these are defined globally and are the ones used by the functions in spinapi*/
IOUSBDeviceInterface **devg = NULL;
IOUSBInterfaceInterface300 	**intfg = NULL;

/* array to hold pid values */
int pid_list[MAX_NUM_BOARDS];

extern int cur_dev;

extern int num_usb_devices;

extern char *spinerr;
// The number of the currently selected board, set to -1 initially.
int cur_board = -1;
// Number of boards present in system. -1 indicates we havent counted them yet - set when os is initialized
int num_boards = -1;

IOReturn ConfigureAnchorDevice(IOUSBDeviceInterface **dev)
{
    UInt8				numConf;
    IOReturn				kr;
    IOUSBConfigurationDescriptorPtr	confDesc;
    
    kr = (*dev)->GetNumberOfConfigurations(dev, &numConf);
    if (!numConf)
        return -1;
    debug("Number of device configurations %d\n",numConf);
    // get the configuration descriptor for index 0
    kr = (*dev)->GetConfigurationDescriptorPtr(dev, 0, &confDesc);
    if (kr)
    {
        debug("\tunable to get config descriptor for index %d (err = %08x)\n", 0, kr);
        return -1;
    }
    kr = (*dev)->SetConfiguration(dev, confDesc->bConfigurationValue);
    if (kr)
    {
        debug("\tunable to set configuration to value %d (err=%08x)\n", 0, kr);
        return -1;
    }
    
    return kIOReturnSuccess;
}


void RawDeviceAdded(void *refCon, io_iterator_t iterator)
{
    kern_return_t		kr;
    io_service_t		usbDevice;
    IOCFPlugInInterface 	**plugInInterface=NULL;
    IOUSBDeviceInterface 	**dev=NULL;
    HRESULT 			res;
    CFNumberRef pid1;
    SInt32 			score, cfvalue, usbProduct = 0xc1a9, usbProduct2 = 0xc2a9, usbProduct3 = 0xc1aa, usbProduct4 = 0x8890;
    UInt16			vendor;
    UInt16			product;
    UInt16			release;
    int exclusiveErr = 0;
    
    while ( (usbDevice = IOIteratorNext(iterator)) )
    {
    pid1 = (CFNumberRef)IORegistryEntryCreateCFProperty(
usbDevice, CFSTR( "idProduct" ), kCFAllocatorDefault, 0 );
	CFNumberGetValue(pid1,kCFNumberSInt32Type,&cfvalue);
	debug("RawDeviceAdded: found pid %d (0x%0x)\n", cfvalue, cfvalue);

	/* we should've gotten all of the pids for a given vid, now see if we have what we want */
	if(cfvalue == usbProduct || cfvalue == usbProduct2 || cfvalue == usbProduct3 || cfvalue == usbProduct4)
	{
		debug("Raw device added.\n");
        kr = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID, kIOCFPlugInInterfaceID, &plugInInterface, &score);
        kr = IOObjectRelease(usbDevice);				// done with the device object now that I have the plugin
        if ((kIOReturnSuccess != kr) || !plugInInterface)
        {
            debug("unable to create a plugin (%08x)\n", kr);
            continue;
        }
            
        // I have the device plugin, I need the device interface
        res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID), (LPVOID)&dev);
        IODestroyPlugInInterface(plugInInterface);			// done with this
		
        if (res || !dev)
        {
            debug("couldn't create a device interface (%08x)\n", (int) res);
            continue;
        }
        // technically should check these kr values
        kr = (*dev)->GetDeviceVendor(dev, &vendor);
        kr = (*dev)->GetDeviceProduct(dev, &product);
        kr = (*dev)->GetDeviceReleaseNumber(dev, &release);

        // need to open the device in order to change its state
        do{
            kr = (*dev)->USBDeviceOpen(dev);
            if(kIOReturnExclusiveAccess == kr)
            {
                exclusiveErr++;
                debug("Exclusive access err, sleeping on it %d\n", exclusiveErr);
                sleep(1);
            }
        }while( (kIOReturnExclusiveAccess == kr) && (exclusiveErr < 5) );
        
        if (kIOReturnSuccess != kr)
        {
            debug("unable to open device: %08x\n", kr);
            (void) (*dev)->Release(dev);
            continue;
        }
        kr = ConfigureAnchorDevice(dev);
        if (kIOReturnSuccess != kr)
        {
            debug("unable to configure device: %08x\n", kr);
            (void) (*dev)->USBDeviceClose(dev);
            (void) (*dev)->Release(dev);
            continue;
        }

        /*stop download to device by maintaining this comment
        
        kr = DownloadToAnchorDevice(dev, vendor, product);
        if (kIOReturnSuccess != kr)
        {
            debug("unable to download to device: %08x\n", kr);
            (void) (*dev)->USBDeviceClose(dev);
            (void) (*dev)->Release(dev);
            continue;
        }
        */

        kr = (*dev)->USBDeviceClose(dev);
        kr = (*dev)->Release(dev);
       }
    }
}

void RawDeviceRemoved(void *refCon, io_iterator_t iterator)
{
    kern_return_t	kr;
    io_service_t	obj;
    
    while ( (obj = IOIteratorNext(iterator)) )
    {
        debug("Raw device removed.\n");
        kr = IOObjectRelease(obj);
    }
}

void get_usb_device_info(io_service_t device)
{

	io_name_t devName;

	io_string_t pathName;

 

	IORegistryEntryGetName(device, devName);

	debug("Device's name = %s\n", devName);

	IORegistryEntryGetPath(device, kIOServicePlane, pathName);

	debug("Device's path in IOService plane = %s\n", pathName);

	IORegistryEntryGetPath(device, kIOUSBPlane, pathName);

	debug("Device's path in IOUSB plane = %s\n", pathName);
}

void NewDeviceAdded(void *refCon, io_iterator_t iterator)
{
    kern_return_t		kr;
    io_service_t		usbDevice;
    IOCFPlugInInterface 	**plugInInterface=NULL;
    IOUSBDeviceInterface **dev = NULL;
    IOUSBDeviceInterface **temp_dev = NULL;
    HRESULT 			res;
    CFNumberRef pid1;
    SInt32 			score, cfvalue, usbProduct = 0xc1a9, usbProduct2 = 0xc2a9, usbProduct3 = 0xc1aa, usbProduct4 = 0x8890;
    UInt16			vendor,product,temp_product,n_devices,i = 0, j = 0;
    UInt32			loc_id, loc_id_tmp, red_loc_id[MAX_NUM_BOARDS], loc_divisor;

    
    while ( (usbDevice = IOIteratorNext(iterator)) )
    {
    pid1 = (CFNumberRef)IORegistryEntryCreateCFProperty(
usbDevice, CFSTR( "idProduct" ), kCFAllocatorDefault, 0 );
	CFNumberGetValue(pid1,kCFNumberSInt32Type,&cfvalue);
	debug("NewDeviceAdded: found pid %d (0x%0x)\n", cfvalue, cfvalue);

	/* we should've gotten all of the pids for a given vid, now see if we have what we want - this is checked here and for the raw device */
	if(cfvalue == usbProduct || cfvalue == usbProduct2 || cfvalue == usbProduct3 || cfvalue == usbProduct4)
	{
        debug("New device added, device %d.\n",i);
        kr = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID, kIOCFPlugInInterfaceID, &plugInInterface, &score);

		/*is there any registry info on this device?*/
		get_usb_device_info(usbDevice);

        kr = IOObjectRelease(usbDevice);				// done with the device object now that I have the plugin
        if ((kIOReturnSuccess != kr) || !plugInInterface)
        {
            debug("unable to create a plugin (%08x)\n", kr);
            continue;
        }
            
        // I have the device plugin, I need the device interface
        res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID), (LPVOID)&dev);
        IODestroyPlugInInterface(plugInInterface);			// done with this
		
        if (res || !dev)
        {
            debug("couldn't create a device interface (%08x)\n", (int) res);
            continue;
        }
        // technically should check these kr values
        kr = (*dev)->GetDeviceVendor(dev, &vendor);
        kr = (*dev)->GetDeviceProduct(dev, &product);
		debug("vendor id is 0x%x, product id is 0x%x\n",vendor,product);
		
		/*find location id*/
		kr = (*dev)->GetLocationID(dev, &loc_id);
		if(kIOReturnSuccess != kr)
			debug("cannot obtain location id for this device!\n");
		else
			{
			/*reduce value of location id; note that the divisor may need to be changed depending on the topology of the usb connections (number of splitters, etc - more splitters means lower divisor, note that different computers may have issues that affect the needed divisor)*/
			loc_divisor = 0x10;
			red_loc_id[i] = loc_id/loc_divisor;
			devg_h[i] = dev;
			pid_list[i] = (int) product;
			debug(" device %d: location id 0x%lx, reduced to 0x%lx\n",i,loc_id,red_loc_id[i-1]);
			i++;
			if ((double)loc_id/(double)red_loc_id[i-1] != (double)loc_divisor)
				debug(" device %d: location id likely reduced too much (ratio %0.8lf)\n",i,(double)loc_id/(double)red_loc_id[i-1]);
			}
		}
     }
    n_devices=i;
    /*number of boards found is set here*/
    num_boards = i;
    /*sort devices by location id, low to high*/
    /*for(i=0; i<num_boards; i++)
    	{
    	kr = (*devg_h[i])->GetDeviceProduct(devg_h[i], &product);
    	printf("before reording boards: board %d, pid 0x%x, actually 0x%x\n",i,pid_list[i],product);
    	}*/
    for(i=0; i<n_devices; i++)
    	{
    	 for(j=0; j<n_devices-1; j++)
    	 {
    	  if(red_loc_id[j] > red_loc_id[j+1])
    	 	{
    	 	 loc_id_tmp = red_loc_id[j+1];
    	 	 temp_dev = devg_h[j+1];
    	 	 temp_product = pid_list[j+1];
    	 	 red_loc_id[j+1] = red_loc_id[j];
    	 	 devg_h[j+1] = devg_h[j];
    	 	 pid_list[j+1] = pid_list[j];
    	 	 red_loc_id[j] = loc_id_tmp;
    	 	 devg_h[j] = temp_dev;
    	 	 pid_list[j] = temp_product;
    	 	 }
    	  }
    	 }
    /*for(i=0; i<num_boards; i++)
    	 {
    	kr = (*devg_h[i])->GetDeviceProduct(devg_h[i], &product);
    	 printf("after reordering boards: board %d, pid 0x%x actually 0x%x\n",i,pid_list[i],product);
    	 }*/
    /*check to see if lowest device location is set*/
    if(!devg_h[0])
       debug("lowest devg_h is not set in NewDeviceAdded\n");

}

void NewDeviceRemoved(void *refCon, io_iterator_t iterator)
{
    kern_return_t	kr;
    io_service_t	obj;
    
    while ( (obj = IOIteratorNext(iterator)) )
    {
        debug("New device removed.\n");
        kr = IOObjectRelease(obj);
    }
}

void SignalHandler(int sigraised)
{
    debug("\nInterrupted\n");
   
    // Clean up here
    IONotificationPortDestroy(gNotifyPort);

    if (gRawAddedIter) 
    {
        IOObjectRelease(gRawAddedIter);
        gRawAddedIter = 0;
    }

    if (gRawRemovedIter) 
    {
        IOObjectRelease(gRawRemovedIter);
        gRawRemovedIter = 0;
    }
    
    if (gNewDeviceAddedIter) 
    {
        IOObjectRelease(gNewDeviceAddedIter);
        gNewDeviceAddedIter = 0;
    }

    if (gNewDeviceRemovedIter) 
    {
        IOObjectRelease(gNewDeviceRemovedIter);
        gNewDeviceRemovedIter = 0;
    }

    // exit(0) should not be called from a signal handler.  Use _exit(0) instead
    //
    _exit(0);
}

IOReturn FindInterfaces(IOUSBDeviceInterface **dev)
{
    IOReturn			kr;
    IOUSBFindInterfaceRequest	request;
    io_iterator_t		iterator;
    io_service_t		usbInterface;
    IOCFPlugInInterface 	**plugInInterface = NULL;
    /* definition requires 10.4 or higher*/
    IOUSBInterfaceInterface300 	**intf = NULL;
   
    HRESULT 			res;
    SInt32 			score;
    UInt8			intfClass;
    UInt8			intfSubClass;
    UInt8			intfNumEndpoints;
    int				pipeRef, n_interfaces = 0;
    
    request.bInterfaceClass = kIOUSBFindInterfaceDontCare;
    request.bInterfaceSubClass = kIOUSBFindInterfaceDontCare;
    request.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
    request.bAlternateSetting = kIOUSBFindInterfaceDontCare;
   
    kr = (*dev)->CreateInterfaceIterator(dev, &request, &iterator);
    
    while ( (usbInterface = IOIteratorNext(iterator)) )
    {
        debug("FindInterfaces: Interface %d on board %d found.\n",n_interfaces++,cur_board);
       
        kr = IOCreatePlugInInterfaceForService(usbInterface, kIOUSBInterfaceUserClientTypeID, kIOCFPlugInInterfaceID, &plugInInterface, &score);
        kr = IOObjectRelease(usbInterface);				// done with the usbInterface object now that I have the plugin
        if ((kIOReturnSuccess != kr) || !plugInInterface)
        {
            debug("unable to create a plugin (%08x)\n", kr);
            break;
        }
            
        // I have the interface plugin. I need the interface interface
        res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBInterfaceInterfaceID), (LPVOID) &intf);
        IODestroyPlugInInterface(plugInInterface);			// done with this

        if (res || !intf)
        {
            debug("couldn't create an IOUSBInterfaceInterface (%08x)\n", (int) res);
            spinerr = "can't open usb device interface";
            break;
        }
        
        kr = (*intf)->GetInterfaceClass(intf, &intfClass);
        kr = (*intf)->GetInterfaceSubClass(intf, &intfSubClass);
        
        debug("Interface class %d, subclass %d\n", intfClass, intfSubClass);
        
        // Now open the interface. This will cause the pipes to be instantiated that are 
        // associated with the endpoints defined in the interface descriptor.
        kr = (*intf)->USBInterfaceOpen(intf);
        if (kIOReturnSuccess != kr)
        {
            debug("unable to open interface (%08x)\n", kr);
            (void) (*intf)->Release(intf);
            spinerr = "can't open usb device interface";
            break;
        }
        
    	kr = (*intf)->GetNumEndpoints(intf, &intfNumEndpoints);
        if (kIOReturnSuccess != kr)
        {
            debug("unable to get number of endpoints (%08x)\n", kr);
            (void) (*intf)->USBInterfaceClose(intf);
            (void) (*intf)->Release(intf);
            break;
        }
        
        debug("Interface has %d endpoints.\n", intfNumEndpoints);
        
        /*set interface array to what we found here*/
        intfg_h[cur_board] = intf;

        for (pipeRef = 1; pipeRef <= intfNumEndpoints; pipeRef++)
        {
            IOReturn	kr2;
            UInt8	direction;
            UInt8	number;
            UInt8	transferType;
            UInt16	maxPacketSize;
            UInt8	interval;
            char	*message;
            
            kr2 = (*intf)->GetPipeProperties(intf, pipeRef, &direction, &number, &transferType, &maxPacketSize, &interval);
            if (kIOReturnSuccess != kr)
                debug("unable to get properties of pipe %d (%08x)\n", pipeRef, kr2);
            else {
                debug("pipeRef %d: ", pipeRef);

                switch (direction) {
                    case kUSBOut:
                        message = "out";
                        break;
                    case kUSBIn:
                        message = "in";
                        break;
                    case kUSBNone:
                        message = "none";
                        break;
                    case kUSBAnyDirn:
                        message = "any";
                        break;
                    default:
                        message = "???";
                }
                debug("direction %s, ", message);
                
                switch (transferType) {
                    case kUSBControl:
                        message = "control";
                        break;
                    case kUSBIsoc:
                        message = "isoc";
                        break;
                    case kUSBBulk:
                        message = "bulk";
                        break;
                    case kUSBInterrupt:
                        message = "interrupt";
                        break;
                    case kUSBAnyType:
                        message = "any";
                        break;
                    default:
                        message = "???";
                }
                debug("transfer type %s, maxPacketSize %d\n", message, maxPacketSize);
            }
        }
        
    }
    
    if(n_interfaces)
    	return kr;
    else
    	return -1;
}


/*the function below is the primary function for setting up the usb interface for the radioprocessor, it is based heavily on existing Apple code, and uses the functions defined above. It includes a signal handler to notify when the device is removed - this is totally useless here, but I left it in since it really has no consequence in doing so*/

int os_usb_init()
{
    mach_port_t 		masterPort;
    CFMutableDictionaryRef 	matchingDict;
    CFRunLoopSourceRef		runLoopSource;
    kern_return_t		kr;
    UInt16				prodID;
    SInt32			usbVendor;//kOurVendorIDs;
    //SInt32 			score, cfvalue, usbProduct = 0xc1a9, usbProduct2 = 0xc2a9, usbProduct3 = 0xc1aa, usbProduct4 = 0x8890; //Product ID;s for SpinCore unused here
    int				num_failures;
    sig_t			oldHandler;

    /*set product and vendor id's to default for possible spincore USB boards - if other product ids are created, they must be added here*/
    usbVendor = 0x0403;
    

    // Set up a signal handler so we can clean up when we're interrupted from the command line
    // Otherwise we stay in our run loop forever.
    oldHandler = signal(SIGINT, SignalHandler);
    if (oldHandler == SIG_ERR)
        debug("Could not establish new signal handler");
        
    // first create a master_port for my task
    kr = IOMasterPort(MACH_PORT_NULL, &masterPort);
    if (kr || !masterPort)
    {
        debug("ERR: Couldn't create a master IOKit Port(%08x)\n", kr);
        return -1;
    }


    // Set up the matching criteria for the devices we're interested in
    matchingDict = IOServiceMatching(kIOUSBDeviceClassName);	// Interested in instances of class IOUSBDevice and its subclasses
    if (!matchingDict)
    {
        debug("Can't create a USB matching dictionary\n");
        mach_port_deallocate(mach_task_self(), masterPort);
        return -1;
    }
    
    /*set up the matching dictionary to match the 3 proeuct id's given at the start of the code - the vendor id is the same in each case */
    
    CFDictionarySetValue( 
            matchingDict, 
            CFSTR(kUSBVendorID), 
            CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbVendor)); 
    /*CFDictionarySetValue( 
            matchingDict, 
            CFSTR(kUSBProductID), 
            CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbProduct));*/ 
    /* match all product ids with this vendor id using a wild card */
    CFDictionarySetValue( 
            matchingDict, 
            CFSTR(kUSBProductID), 
            CFSTR("*")); 

	/* print matching dictionary */
	/*CFShow(matchingDict);*/
	
    // Create a notification port and add its run loop event source to our run loop
    // This is how async notifications get set up.
    gNotifyPort = IONotificationPortCreate(masterPort);
    runLoopSource = IONotificationPortGetRunLoopSource(gNotifyPort);
    
    CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource, kCFRunLoopDefaultMode);
    
    // Retain additional references because we use this same dictionary with four calls to 
    // IOServiceAddMatchingNotification, each of which consumes one reference.
    matchingDict = (CFMutableDictionaryRef) CFRetain( matchingDict ); 
    matchingDict = (CFMutableDictionaryRef) CFRetain( matchingDict ); 
    matchingDict = (CFMutableDictionaryRef) CFRetain( matchingDict ); 
    
    // Now set up two notifications, one to be called when a raw device is first matched by I/O Kit, and the other to be
    // called when the device is terminated.
    kr = IOServiceAddMatchingNotification(  gNotifyPort,
                                            kIOFirstMatchNotification,
                                            matchingDict,
                                            RawDeviceAdded,
                                            NULL,
                                            &gRawAddedIter );
                                            
    RawDeviceAdded(NULL, gRawAddedIter);	// Iterate once to get already-present devices and
                                                // arm the notification

    kr = IOServiceAddMatchingNotification(  gNotifyPort,
                                            kIOTerminatedNotification,
                                            matchingDict,
                                            RawDeviceRemoved,
                                            NULL,
                                            &gRawRemovedIter );
                                            
    RawDeviceRemoved(NULL, gRawRemovedIter);	// Iterate once to arm the notification
    

    // Now set up two more notifications, one to be called when a new test device is first matched by I/O Kit, and the other to be
    // called when the device is terminated.
    kr = IOServiceAddMatchingNotification(  gNotifyPort,
                                            kIOFirstMatchNotification,
                                            matchingDict,
                                            NewDeviceAdded,
                                            NULL,
                                            &gNewDeviceAddedIter );
                                            
    NewDeviceAdded(NULL, gNewDeviceAddedIter);	// Iterate once to get already-present devices and arm the notification
    
    kr = IOServiceAddMatchingNotification(  gNotifyPort,
                                            kIOTerminatedNotification,
                                            matchingDict,
                                            NewDeviceRemoved,
                                            NULL,
                                            &gNewDeviceRemovedIter );
                                            
    NewDeviceRemoved(NULL, gNewDeviceRemovedIter); 	// Iterate once to arm the notification

    // Now done with the master_port
    mach_port_deallocate(mach_task_self(), masterPort);
    masterPort = 0;

    /*see if we have any devices, should have at least one*/
    if(!devg_h[0])
    	{
    	 debug("no devices found!\n");
    	 spinerr = "can't open usb device";
    	 return -1;
    	 }
    /*see if we have any device interfaces - try selecting board 0*/
	/*set global device pointer to desired device*/
	num_failures = 0;
	for(cur_board=0; cur_board<num_boards; cur_board++)
	{
	 /*printf("looking for device interface %d pid 0x%x\n",cur_board,pid_list[cur_board]);*/
	 devg = devg_h[cur_board];
	 /*see if we have any device interfaces*/
	 if(!devg)
    	{
    	 debug("devg is not set when selecting board %d\n",cur_board);
    	 spinerr = "can't open usb device";
    	 }
     else
    	{
        	kr = FindInterfaces(devg);
        	if (kIOReturnSuccess != kr)
        	{
            	spinerr = "can't open usb device";
				debug("unable to find interfaces on device: %08x\n", kr);
            	num_failures++;
            	}
		}
	 }
	if(num_failures != 0)
		{
		 return -1;
		 spinerr = "unable to open a board interface in os_init";
		}
	/* see what the interfaces correspond to */
	num_usb_devices = num_boards;
	for(cur_board=0; cur_board<num_boards; cur_board++)
		{
		 kr = (*intfg_h[cur_board])->GetDeviceProduct(intfg_h[cur_board],&prodID);
		 debug2("at end of os_init, for board %d prod id is 0x%x, pid array is 0x%x\n",cur_board,prodID,pid_list[cur_board]);
		 }
	/*by default, select board 0*/
	if(pb_select_board(0))
		return (int)pid_list[0];
	else
		{
		 return -1;
		 spinerr = "unable to select board 0 in os_init";
		}
}


/*select board by opening interface and setting global interface - this function is placed here since it uses the non-global interface array*/
int pb_select_board (int board_num)
{
	/*all this does is set the global device interface to the array value for the selected board*/
	/*set global device pointer to desired device*/
	debug("selecting board %d, pid 0x%x\n",board_num,pid_list[board_num]);
	intfg = intfg_h[board_num];
	/*see if we have any device interfaces*/
	if(intfg)
		{
		 cur_board = board_num;
		 /* cur_dev is used in some spincore code */
		 cur_dev = board_num;
		 return 1;
		 }
	else
		{
		 return -1;
		 spinerr = "global interface pointer is not set";
		 debug("global interface pointer is not set\n");
		}
}
